module set_max_speed_m

  implicit none

contains

  subroutine set_max_speed(e, ind_targ_extr, outside_points, ssh, u, v, &
       corner, step)

    ! This procedure defines the components speed_cont, max_speed and
    ! radius4 of argument e. On return, e%speed_cont may be a null ssh
    ! contour but e%max_speed is never set to missing_speed. However,
    ! e%max_speed might be NaN if speed_outerm is NaN (with any value
    ! of e%radius4). The value of e%radius4 set here is always >=
    ! 1. On return, if e%speed_cont is not a null ssh contour then
    ! e%max_speed is the speed on e%speed_cont.

    ! The length of the interval of longitudes of the domain, step(1)
    ! * (size(ssh, 1) - 1), should be < 180 degrees, so that the
    ! geometrical processing done here is non-ambiguous. The
    ! longitudes of outside points should be in the interval
    ! [corner(1), corner(1) + step(1) * (size(ssh, 1) - 1)] as there
    ! will be no automatic shifting by a multiple of 360 degrees.

    use, intrinsic:: IEEE_ARITHMETIC, only: IEEE_IS_NAN

    ! Lbraries:
    use contour_531, only: convert_to_ind
    use geometry, only: polygon_point_dist_2d

    use ccw_orient_m, only: ccw_orient
    use derived_types, only: eddy, null_ssh_contour, missing_ssh
    use good_contour_m, only: good_contour
    use inside_4_m, only: inside_4
    use max_speed_contour_ssh_m, only: max_speed_contour_ssh
    use mean_speed_m, only: mean_speed
    use spher_polyline_area_m, only: spher_polyline_area

    type(eddy), intent(inout):: e
    ! e%out_cont, e%cyclone, e%extr%ssh and e%extr%coord
    ! should be defined on input.

    integer, intent(in):: ind_targ_extr(:) ! (2) indices of the target extremum

    real, intent(in):: outside_points(:, :) ! (2, :) longitude and
    ! latitude, in rad, of all the extrema except the target extremum

    real, intent(in):: ssh(:, :), u(:, :), v(:, :)
    ! The domain is allowed to be as small as the bounding box of
    ! out_cont. If the domain is larger, this subroutine will work
    ! too, but with wasted computation.

    real, intent(in):: corner(:) ! (2)
    ! longitude and latitude of the lower left corner of the grid, in rad

    real, intent(in):: step(:) ! (2) longitude and latitude steps, in rad

    ! Local:
    real speed_outerm ! speed on the outermost contour

    !---------------------------------------------------------------

    ! Compute lower bound of e%radius4 from distance in index space
    ! from extremum to outermost contour:
    e%radius4 = floor(polygon_point_dist_2d(convert_to_ind(e%out_cont%points, &
         corner, step), real(ind_targ_extr))) + 1

    do while (inside_4(e%radius4 * step, e%extr%coord, e%out_cont%points))
       e%radius4 = e%radius4 + 1
    end do

    ! Might the outermost contour be a better choice?
    speed_outerm = mean_speed(u, v, e%out_cont%polyline, e%extr%coord, corner, &
         step)

    if (e%radius4 == 1) then
       ! We cannot look for a maximum speed contour other than out_cont.
       e%speed_cont = null_ssh_contour()

       e%max_speed = speed_outerm
       ! (We will maybe need this to compute the weight of edges in
       ! the tracking graph.)
    else
       ! {e%radius4 >= 2}

       e%speed_cont%ssh = max_speed_contour_ssh(ssh, u, v, ind_targ_extr, &
            e%radius4)

       if (e%speed_cont%ssh == missing_ssh) then
          e%speed_cont = null_ssh_contour()
          e%max_speed = speed_outerm
       else
          e%speed_cont%polyline = good_contour(corner, step, ssh, &
               e%speed_cont%ssh, e%extr%coord, outside_points)

          if (e%speed_cont%n_points == 0) then
             ! This may happen if there is the exact same value of ssh
             ! next to the extremum and this value is
             ! e%speed_cont%ssh.
             e%speed_cont%ssh = missing_ssh
             e%speed_cont%area = - 1e6
             e%max_speed = speed_outerm
          else
             e%max_speed = mean_speed(u, v, e%speed_cont%polyline, &
                  e%extr%coord, corner, step)

             if (IEEE_IS_NAN(speed_outerm)) then
                ! This may happen when the outermost contour is near land.
                ! Stick to the contour coming from max_speed_contour_ssh.
                e%speed_cont%area = spher_polyline_area(e%speed_cont%polyline)
                call ccw_orient(e%speed_cont)
             else
                ! Note the following test should raise an invalid exception
                ! if e%max_speed is NaN.
                if (abs(speed_outerm) > abs(e%max_speed)) then
                   ! Abandon the contour coming from max_speed_contour_ssh:
                   e%speed_cont = null_ssh_contour()
                   e%max_speed = speed_outerm
                else
                   ! Stick to the contour coming from max_speed_contour_ssh.
                   e%speed_cont%area &
                        = spher_polyline_area(e%speed_cont%polyline)
                   call ccw_orient(e%speed_cont)
                end if
             end if
          end if
       end if
    end if

  end subroutine set_max_speed

end module set_max_speed_m
